﻿using System;
using System.Diagnostics;
using System.Windows;
using StatsTracker.Reporters;
using StatsTracker.Trackers;
using StatsTracker.UI;
using StatsTracker.Util;
using Zeta.Bot;
using Zeta.Common.Plugins;
using Zeta.Game;

namespace StatsTracker
{
    public class DamageTracker : IPlugin
    {

        public string Name { get; private set; }
        public Version Version { get; private set; }
        public string Author { get; private set; }
        public string Description { get; private set; }
        public Window DisplayWindow { get { return null; } }


        internal const string PluginName = "Damage Tracker";
        internal static Version PluginVersion = new Version(0, 2, 50615);
        internal static bool IsBotStarted = false;

        private long _lastUpdate;

        public void OnPulse()
        {
            if (!IsBotStarted)
            {
                return;
            }

            if (IsGameOrBotPaused)
            {
                return;
            }

            if (ZetaDia.Me == null || ZetaDia.Me.CommonData == null || !ZetaDia.Me.IsValid || !ZetaDia.Me.CommonData.IsValid)
                return;

            if (!ZetaDia.IsInGame || ZetaDia.IsLoadingWorld || ZetaDia.IsPlayingCutscene || ZetaDia.IsInTown)
                return;

            UnitTracker.TrackUnits();
            var currentSecond = TimeProvider.CurrentSecond;
            if (_lastUpdate != currentSecond)
            {
                DamageReporter.UpdateStats();
                _lastUpdate = currentSecond;
            }
            DamageReporter.ReportCurrentDps();

        }

        public void OnInitialize()
        {
            Logger.Info(string.Format("({0}) initialized.", Version));
            BotMain.OnStart += OnBotStart;
            BotMain.OnStop += OnBotStop;

        }

        public void OnShutdown()
        {

        }

        public void OnEnabled()
        {
            TabUi.InstallTab();
            Logger.Info(string.Format("({0}) enabled.", Version));
        }

        public void OnDisabled()
        {
            TabUi.RemoveTab();
            Logger.Info((string.Format("({0}) disabled.", Version)));
        }

        internal static void OnBotStart(IBot bot)
        {
            IsBotStarted = true;
        }

        internal static void OnBotStop(IBot bot)
        {
            IsBotStarted = false;
        }


        private DamageTracker()
        {
            Name = PluginName;
            Version = PluginVersion;
            Author = "TarasBulba";
            Description = "A plugin for tracking damage statistics";
        }

        public bool Equals(IPlugin other)
        {
            return false;
        }

        internal static bool IsGameOrBotPaused
        {
            get
            {
                if (!string.IsNullOrWhiteSpace(BotMain.StatusText))
                {
                    return BotMain.StatusText.IndexOf("paused", StringComparison.InvariantCultureIgnoreCase) > 0;
                }
                return true;
            }
        }


    }

}
